@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2018 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2018 Laszlo Molnar
+   Copyright (C) 1996-2020 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2020 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -155,12 +155,14 @@
         DT_RELA     =  7,       /* Relocations which do contain an addend */
         DT_RELASZ   =  8,       /* Total size of Rela relocs */
         DT_RELAENT  =  9,       /* Size of one RELA relocation */
+        DT_STRSZ    = 10,       /* Size of string table */
+        DT_SYMENT   = 11,       /* Size of one symbol table entry */
         DT_INIT     = 12,       /* Address of init function */
         DT_FINI     = 13,       /* Address of termination function */
+
         DT_REL      = 17,       /* Relocations which contain no addend */
         DT_RELSZ   =  18,       /* Total size of Rel relocs */
         DT_RELENT   = 19,       /* Size of one Rel relocation */
-        DT_STRSZ    = 10,       /* Sizeof string table */
         DT_PLTREL   = 20,       /* Type of reloc in PLT */
         DT_TEXTREL  = 22,       /* Reloc might modify .text */
         DT_JMPREL   = 23,       /* Address of PLT relocs */
@@ -240,6 +242,13 @@
         R_PPC_JMP_SLOT = 21,
         R_PPC64_JMP_SLOT = R_PPC_JMP_SLOT,
         R_X86_64_JUMP_SLOT = 7
+
+        , R_ARM_ABS32 = 2
+        , R_ARM_GLOB_DAT = 21
+
+        , R_AARCH64_ABS64 = 257
+        , R_AARCH64_GLOB_DAT = 1025
+
     };
 #endif  //}
 
